# pylint: disable=no-member
from functools import partial
from unittest import mock, TestCase

from pcs_test.tier0.lib.commands.remote_node.fixtures_add import(
    EnvConfigMixin,
    REPORTS as FIXTURE_REPORTS,
    EXTRA_REPORTS as FIXTURE_EXTRA_REPORTS,
    FAIL_HTTP_KWARGS,
    fixture_reports_not_live_cib,
    fixture_reports_new_node_unreachable,
)
from pcs_test.tools import fixture
from pcs_test.tools.command_env import get_env_tools
from pcs_test.tools.misc import get_test_resource as rc
from pcs_test.tools.xml import XmlManipulation

from pcs import settings
from pcs.common import report_codes
from pcs.common.host import Destination
from pcs.lib.commands.remote_node import node_add_guest as node_add_guest_orig


NODE_NAME = "node-name"
NODE_ADDR_PCSD = "node-addr"
NODE_DEST_LIST = [Destination(NODE_ADDR_PCSD, 2224)]
REMOTE_HOST = "remote-host"
VIRTUAL_MACHINE_ID = "virtual_machine_id"
NODE_1 = "node-1"
NODE_1_DEST_LIST = [Destination("node-1-addr", 2224)]
NODE_2 = "node-2"
NODE_2_DEST_LIST = [Destination("node-2-addr", 2224)]
KNOWN_HOSTS_DESTS = {
    NODE_NAME: NODE_DEST_LIST,
    NODE_1: NODE_1_DEST_LIST,
    NODE_2: NODE_2_DEST_LIST,
}

def node_add_guest(
    env, node_name=NODE_NAME, resource_id=VIRTUAL_MACHINE_ID, options=None,
    **kwargs
):
    options = options or {"remote-addr": REMOTE_HOST}
    node_add_guest_orig(env, node_name, resource_id, options, **kwargs)

FIXTURE_RESOURCES = """
    <resources>
        <primitive class="ocf" id="{0}"
            provider="heartbeat" type="VirtualDomain"
        />
    </resources>
""".format(VIRTUAL_MACHINE_ID)

FIXTURE_META_ATTRIBUTES = """
    <meta_attributes id="virtual_machine_id-meta_attributes">
        <nvpair id="virtual_machine_id-meta_attributes-remote-addr"
            name="remote-addr" value="remote-host"
        />
        <nvpair id="virtual_machine_id-meta_attributes-remote-node"
            name="remote-node" value="node-name"
        />
     </meta_attributes>
"""


class LocalConfig(EnvConfigMixin):
    def load_cib(self):
        self.config.runner.cib.load(resources=FIXTURE_RESOURCES)

    def push_cib(self, wait=False, meta_attributes=FIXTURE_META_ATTRIBUTES):
        self.config.env.push_cib(
            append={
                 './/resources/primitive[@id="{0}"]'
                    .format(VIRTUAL_MACHINE_ID): meta_attributes
                ,
            },
            wait=wait
        )

get_env_tools = partial(get_env_tools, local_extensions={"local": LocalConfig})

def base_reports_for_host(host=NODE_NAME):
    return (
        FIXTURE_REPORTS
            .adapt("authkey_distribution_started", node_list=[host])
            .adapt("authkey_distribution_success", node=host)
            .adapt("pcmk_remote_start_enable_started", node_list=[host])
            .adapt("pcmk_remote_enable_success", node=host)
            .adapt("pcmk_remote_start_success", node=host)
    )

REPORTS = base_reports_for_host()

EXTRA_REPORTS = (FIXTURE_EXTRA_REPORTS.adapt_multi(
    [
        "manage_services_connection_failed",
        "manage_services_connection_failed_warn",
        "check_availability_connection_failed",
        "check_availability_connection_failed_warn",
        "put_file_connection_failed",
        "put_file_connection_failed_warn",
        "pcmk_remote_enable_failed",
        "pcmk_remote_enable_failed_warn",
        "pcmk_remote_start_failed",
        "pcmk_remote_start_failed_warn",
        "authkey_distribution_failed",
        "authkey_distribution_failed_warn",
    ],
    node=NODE_NAME
))

class AddGuest(TestCase):
    def setUp(self):
        self.env_assist, self.config = get_env_tools(self)
        self.config.env.set_known_hosts_dests(KNOWN_HOSTS_DESTS)

    def _config_success_base(self):
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .http.host.check_auth(
                communication_list=[
                    dict(label=NODE_NAME, dest_list=NODE_DEST_LIST)
                ],
            )
            .local.get_host_info(NODE_NAME, NODE_DEST_LIST)
            .local.push_existing_authkey_to_remote(NODE_NAME, NODE_DEST_LIST)
            .local.run_pacemaker_remote(NODE_NAME, NODE_DEST_LIST)
            .local.push_cib()
        )

    def test_success_base(self):
        self._config_success_base()
        node_add_guest(self.env_assist.get_env())
        self.env_assist.assert_reports(REPORTS)

    @mock.patch("pcs.lib.commands.remote_node.generate_binary_key")
    def test_success_generated_authkey(self, generate_binary_key):
        generate_binary_key.return_value = b"password"
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .http.host.check_auth(
                communication_list=[
                    dict(label=NODE_NAME, dest_list=NODE_DEST_LIST)
                ],
            )
            .local.get_host_info(NODE_NAME, NODE_DEST_LIST)
            .local.authkey_exists(return_value=False)
            .local.distribute_authkey(
                communication_list=[
                    dict(label=NODE_1, dest_list=NODE_1_DEST_LIST),
                    dict(label=NODE_2, dest_list=NODE_2_DEST_LIST),
                    dict(label=NODE_NAME, dest_list=NODE_DEST_LIST),
                ],
                pcmk_authkey_content=generate_binary_key.return_value,
            )
            .local.run_pacemaker_remote(NODE_NAME, NODE_DEST_LIST)
            .local.push_cib()
        )
        node_add_guest(self.env_assist.get_env())
        generate_binary_key.assert_called_once_with(random_bytes_count=256)
        self.env_assist.assert_reports(
            REPORTS
                .adapt(
                    "authkey_distribution_started",
                    node_list=[NODE_1, NODE_2, NODE_NAME]
                )
                .copy(
                    "authkey_distribution_success",
                    "authkey_distribution_success_node1",
                    node=NODE_1,
                )
                .copy(
                    "authkey_distribution_success",
                    "authkey_distribution_success_node2",
                    node=NODE_2,
                )
        )

    def test_new_offline(self):
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .http.host.check_auth(
                communication_list=[
                    dict(
                        label=NODE_NAME,
                        dest_list=NODE_DEST_LIST,
                        **FAIL_HTTP_KWARGS
                    )
                ],
            )
        )
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(self.env_assist.get_env()),
            []
        )
        self.env_assist.assert_reports([
            fixture.error(
                report_codes.NODE_COMMUNICATION_ERROR_UNABLE_TO_CONNECT,
                force_code=report_codes.SKIP_OFFLINE_NODES,
                node=NODE_NAME,
                command="remote/check_auth",
                reason="Could not resolve host",
            )
        ])

    def test_can_skip_new_offline(self):
        pcmk_authkey_content = b"password"
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .http.host.check_auth(
                communication_list=[
                    dict(
                        label=NODE_NAME,
                        dest_list=NODE_DEST_LIST,
                        **FAIL_HTTP_KWARGS
                    )
                ],
            )
            .local.authkey_exists(return_value=True)
            .local.open_authkey(pcmk_authkey_content)
            .local.push_cib()
        )
        node_add_guest(self.env_assist.get_env(), skip_offline_nodes=True)
        self.env_assist.assert_reports(
            fixture_reports_new_node_unreachable(NODE_NAME, omitting=True)
        )

    @mock.patch("pcs.lib.commands.remote_node.generate_binary_key")
    def test_can_skip_all_offline(self, generate_binary_key):
        generate_binary_key.return_value = b"password"
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .http.host.check_auth(
                communication_list=[
                    dict(
                        label=NODE_NAME,
                        dest_list=NODE_DEST_LIST,
                        **FAIL_HTTP_KWARGS
                    )
                ],
            )
            .local.authkey_exists(return_value=False)
            .local.distribute_authkey(
                communication_list=[
                    dict(
                        label=NODE_1,
                        dest_list=NODE_1_DEST_LIST,
                        **FAIL_HTTP_KWARGS,
                    ),
                    dict(
                        label=NODE_2,
                        dest_list=NODE_2_DEST_LIST,
                        **FAIL_HTTP_KWARGS,
                    ),
                ],
                pcmk_authkey_content=generate_binary_key.return_value,
            )
            .local.push_cib()
        )
        node_add_guest(self.env_assist.get_env(), skip_offline_nodes=True)
        self.env_assist.assert_reports(
            fixture_reports_new_node_unreachable(NODE_NAME, omitting=True)
            + [
            fixture.info(
                report_codes.FILES_DISTRIBUTION_STARTED,
                file_list=["pacemaker authkey"],
                node_list=[NODE_1, NODE_2],
            ),
            fixture.warn(
                report_codes.NODE_COMMUNICATION_ERROR_UNABLE_TO_CONNECT,
                node=NODE_1,
                command="remote/put_file",
                reason="Could not resolve host",
            ),
            fixture.warn(
                report_codes.NODE_COMMUNICATION_ERROR_UNABLE_TO_CONNECT,
                node=NODE_2,
                command="remote/put_file",
                reason="Could not resolve host",
            ),
        ])

    def test_fails_when_remote_node_is_not_prepared(self):
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .http.host.check_auth(
                communication_list=[
                    dict(label=NODE_NAME, dest_list=NODE_DEST_LIST)
                ],
            )
            .local.get_host_info(
                NODE_NAME,
                NODE_DEST_LIST,
                output=dict(
                    services=dict(
                        pacemaker_remote=dict(
                            installed=False, enabled=False, running=False
                        ),
                        pacemaker=dict(
                            installed=True, enabled=False, running=True
                        ),
                        corosync=dict(
                            installed=True, enabled=False, running=True
                        ),
                    ),
                    cluster_configuration_exists=True,
                )
            )
        )
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(self.env_assist.get_env()),
            []
        )
        self.env_assist.assert_reports([
            fixture.error(
                report_codes.SERVICE_NOT_INSTALLED,
                node=NODE_NAME,
                service_list=["pacemaker_remote"],
            ),
            fixture.error(
                report_codes.HOST_ALREADY_IN_CLUSTER_SERVICES,
                host_name=NODE_NAME,
                service_list=["corosync", "pacemaker"],
            ),
            fixture.error(
                report_codes.HOST_ALREADY_IN_CLUSTER_CONFIG,
                host_name=NODE_NAME,
            ),
        ])

    def test_fails_when_remote_node_returns_invalid_output(self):
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .http.host.check_auth(
                communication_list=[
                    dict(label=NODE_NAME, dest_list=NODE_DEST_LIST)
                ],
            )
            .local.get_host_info(
                NODE_NAME, NODE_DEST_LIST, output="INVALID_OUTPUT"
            )
        )
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(self.env_assist.get_env()),
            []
        )
        self.env_assist.assert_reports([
            fixture.error(
                report_codes.INVALID_RESPONSE_FORMAT,
                node=NODE_NAME,
            )
        ])

    def test_changed_options(self):
        meta_attributes = """
            <meta_attributes id="virtual_machine_id-meta_attributes">
                <nvpair id="virtual_machine_id-meta_attributes-remote-addr"
                    name="remote-addr" value="node-addr"
                />
                <nvpair
                    id="virtual_machine_id-meta_attributes-remote-connect-timeout"
                    name="remote-connect-timeout" value="20"
                />
                <nvpair
                    id="virtual_machine_id-meta_attributes-remote-node"
                    name="remote-node" value="node-name"
                />
                <nvpair
                    id="virtual_machine_id-meta_attributes-remote-port"
                    name="remote-port" value="1234"
                />
            </meta_attributes>
        """
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .http.host.check_auth(
                communication_list=[
                    dict(label=NODE_NAME, dest_list=NODE_DEST_LIST)
                ],
            )
            .local.get_host_info(NODE_NAME, NODE_DEST_LIST)
            .local.push_existing_authkey_to_remote(NODE_NAME, NODE_DEST_LIST)
            .local.run_pacemaker_remote(NODE_NAME, NODE_DEST_LIST)
            .local.push_cib(meta_attributes=meta_attributes)
        )
        # Since options are set, the default remote-addr == REMOTE_HOST is not
        # set. Therefore, it defaults to node name which is NODE_NAME.
        node_add_guest(self.env_assist.get_env(), options={
            "remote-port": 1234,
            "remote-connect-timeout": 20
        })
        self.env_assist.assert_reports(
            base_reports_for_host(NODE_NAME).reports
            +
            [
                fixture.info(
                    report_codes.USING_KNOWN_HOST_ADDRESS_FOR_HOST,
                    host_name=NODE_NAME,
                    address=NODE_ADDR_PCSD,
                )
            ]
        )

    def test_noexistent_resource(self):
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
        )
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(
                self.env_assist.get_env(),
                resource_id="NOEXISTENT"
            ),
            []
        )
        self.env_assist.assert_reports(
            [
                fixture.error(
                    report_codes.ID_NOT_FOUND,
                    expected_types=["primitive"],
                    context_type="resources",
                    id="NOEXISTENT",
                    context_id=""
                )
            ],
        )

    def test_validate_values(self):
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
        )
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(
                self.env_assist.get_env(),
                options={
                    "remote-addr": "*addr",
                    "remote-port": "abc",
                    "remote-connect-timeout": "def",
                }
            ),
            []
        )
        self.env_assist.assert_reports(
            [
                fixture.error(
                    report_codes.INVALID_OPTION_VALUE,
                    option_name="remote-connect-timeout",
                    option_value="def",
                    allowed_values="time interval (e.g. 1, 2s, 3m, 4h, ...)",
                    cannot_be_empty=False,
                    forbidden_characters=None,
                ),
                fixture.error(
                    report_codes.INVALID_OPTION_VALUE,
                    option_name="remote-port",
                    option_value="abc",
                    allowed_values="a port number (1..65535)",
                    cannot_be_empty=False,
                    forbidden_characters=None,
                )
            ]
        )

    def test_unknown_host(self):
        self.config.env.set_known_hosts_dests({
            NODE_1: NODE_1_DEST_LIST,
            NODE_2: NODE_2_DEST_LIST,
        })
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
        )
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(self.env_assist.get_env()),
            []
        )
        self.env_assist.assert_reports([
            fixture.error(
                report_codes.HOST_NOT_FOUND,
                force_code=report_codes.SKIP_OFFLINE_NODES,
                host_list=[NODE_NAME],
            )
        ])

    def test_unknown_host_skip_offline(self):
        pcmk_authkey_content = b"password"
        self.config.env.set_known_hosts_dests({
            NODE_1: NODE_1_DEST_LIST,
            NODE_2: NODE_2_DEST_LIST,
        })
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .local.authkey_exists(return_value=True)
            .local.open_authkey(pcmk_authkey_content)
            .local.push_cib()
        )
        node_add_guest(self.env_assist.get_env(), skip_offline_nodes=True)
        self.env_assist.assert_reports(
            fixture_reports_new_node_unreachable(NODE_NAME)
        )

    @mock.patch("pcs.lib.commands.remote_node.generate_binary_key")
    def test_unknown_host_skip_offline_authkey_distribution(
        self, generate_binary_key
    ):
        generate_binary_key.return_value = b"password"
        self.config.env.set_known_hosts_dests({
            NODE_1: NODE_1_DEST_LIST,
            NODE_2: NODE_2_DEST_LIST,
        })
        (self.config
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .local.authkey_exists(return_value=False)
            .local.distribute_authkey(
                communication_list=[
                    dict(label=NODE_1, dest_list=NODE_1_DEST_LIST),
                    dict(label=NODE_2, dest_list=NODE_2_DEST_LIST),
                ],
                pcmk_authkey_content=generate_binary_key.return_value,
            )
            .local.push_cib()
        )
        node_add_guest(self.env_assist.get_env(), skip_offline_nodes=True)
        generate_binary_key.assert_called_once_with(random_bytes_count=256)
        self.env_assist.assert_reports(
            fixture_reports_new_node_unreachable(NODE_NAME)
            + [
            fixture.info(
                report_codes.FILES_DISTRIBUTION_STARTED,
                file_list=["pacemaker authkey"],
                node_list=[NODE_1, NODE_2],
            ),
            fixture.info(
                report_codes.FILE_DISTRIBUTION_SUCCESS,
                file_description="pacemaker authkey",
                node=NODE_1,
            ),
            fixture.info(
                report_codes.FILE_DISTRIBUTION_SUCCESS,
                file_description="pacemaker authkey",
                node=NODE_2,
            ),
        ])

    def test_some_node_names_missing(self):
        self._config_success_base()
        (self.config
            .env.set_known_hosts_dests({
                "rh7-1": [Destination("rh7-1", 2224)],
                "rh7-2": [Destination("rh7-2", 2224)],
                NODE_NAME: NODE_DEST_LIST,
            })
            .corosync_conf.load(
                filename="corosync-some-node-names.conf",
                instead="corosync_conf.load"
            )
        )

        node_add_guest(self.env_assist.get_env())
        self.env_assist.assert_reports(
            REPORTS.warn(
                "missing_node_names_in_corosync",
                report_codes.COROSYNC_CONFIG_MISSING_NAMES_OF_NODES,
                fatal=False
            )
        )

    def test_all_node_names_missing(self):
        self._config_success_base()
        self.config.corosync_conf.load(
            filename="corosync-no-node-names.conf",
            instead="corosync_conf.load"
        )
        node_add_guest(self.env_assist.get_env())
        self.env_assist.assert_reports(
            REPORTS.warn(
                "missing_node_names_in_corosync",
                report_codes.COROSYNC_CONFIG_MISSING_NAMES_OF_NODES,
                fatal=False
            )
        )


class NotLive(TestCase):
    def setUp(self):
        self.env_assist, self.config = get_env_tools(self)
        self.config.env.set_known_hosts_dests(KNOWN_HOSTS_DESTS)
        cib_xml_man = XmlManipulation.from_file(rc("cib-empty.xml"))
        cib_xml_man.append_to_first_tag_name(
            "resources",
            """
                <primitive class="ocf" id="{0}"
                    provider="heartbeat" type="VirtualDomain"
                />
            """.format(VIRTUAL_MACHINE_ID)
        )
        self.config.env.set_cib_data(str(cib_xml_man))

    def test_addr_specified(self):
        (self.config
            .local.load_cib()
            .local.push_cib()
        )
        node_add_guest(self.env_assist.get_env())
        self.env_assist.assert_reports(fixture_reports_not_live_cib(NODE_NAME))

    def test_addr_not_specified(self):
        meta_attributes = """
            <meta_attributes id="virtual_machine_id-meta_attributes">
                <nvpair id="virtual_machine_id-meta_attributes-remote-addr"
                    name="remote-addr" value="node-addr"
                />
                <nvpair
                    id="virtual_machine_id-meta_attributes-remote-node"
                    name="remote-node" value="node-name"
                />
                <nvpair
                    id="virtual_machine_id-meta_attributes-remote-port"
                    name="remote-port" value="99"
                />
            </meta_attributes>
        """
        (self.config
            .local.load_cib()
            .local.push_cib(meta_attributes=meta_attributes)
        )
        node_add_guest(self.env_assist.get_env(), options={"remote-port": "99"})
        self.env_assist.assert_reports(
            [
                fixture.info(
                    report_codes.USING_KNOWN_HOST_ADDRESS_FOR_HOST,
                    host_name=NODE_NAME,
                    address=NODE_ADDR_PCSD,
                ),
            ]
            +
            fixture_reports_not_live_cib(NODE_NAME)
        )

    def test_unknown_host_addr_not_specified(self):
        meta_attributes = """
            <meta_attributes id="virtual_machine_id-meta_attributes">
                <nvpair id="virtual_machine_id-meta_attributes-remote-addr"
                    name="remote-addr" value="node-name"
                />
                <nvpair
                    id="virtual_machine_id-meta_attributes-remote-node"
                    name="remote-node" value="node-name"
                />
                <nvpair
                    id="virtual_machine_id-meta_attributes-remote-port"
                    name="remote-port" value="99"
                />
            </meta_attributes>
        """
        self.config.env.set_known_hosts_dests(dict())
        (self.config
            .local.load_cib()
            .local.push_cib(meta_attributes=meta_attributes)
        )
        node_add_guest(self.env_assist.get_env(), options={"remote-port": "99"})
        self.env_assist.assert_reports(
            [
                fixture.info(
                    report_codes.USING_KNOWN_HOST_ADDRESS_FOR_HOST,
                    host_name=NODE_NAME,
                    address=NODE_NAME,
                ),
            ]
            +
            fixture_reports_not_live_cib(NODE_NAME)
        )

    def test_unknown_host_addr_specified(self):
        meta_attributes = """
            <meta_attributes id="virtual_machine_id-meta_attributes">
                <nvpair id="virtual_machine_id-meta_attributes-remote-addr"
                    name="remote-addr" value="aa"
                />
                <nvpair
                    id="virtual_machine_id-meta_attributes-remote-node"
                    name="remote-node" value="node-name"
                />
            </meta_attributes>
        """
        self.config.env.set_known_hosts_dests(dict())
        (self.config
            .local.load_cib()
            .local.push_cib(meta_attributes=meta_attributes)
        )
        node_add_guest(self.env_assist.get_env(), options={"remote-addr": "aa"})
        self.env_assist.assert_reports(fixture_reports_not_live_cib(NODE_NAME))

    def test_validate_values(self):
        (self.config
            .local.load_cib()
        )
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(
                self.env_assist.get_env(),
                options={
                    "remote-addr": "*addr",
                    "remote-port": "abc",
                    "remote-connect-timeout": "def",
                }
            ),
            []
        )
        self.env_assist.assert_reports([
            fixture.info(
                report_codes.COROSYNC_NODE_CONFLICT_CHECK_SKIPPED,
                reason_type="not_live_cib",
            ),
            fixture.error(
                report_codes.INVALID_OPTION_VALUE,
                option_name="remote-connect-timeout",
                option_value="def",
                allowed_values="time interval (e.g. 1, 2s, 3m, 4h, ...)",
                    cannot_be_empty=False,
                    forbidden_characters=None,
            ),
            fixture.error(
                report_codes.INVALID_OPTION_VALUE,
                option_name="remote-port",
                option_value="abc",
                allowed_values="a port number (1..65535)",
                    cannot_be_empty=False,
                    forbidden_characters=None,
            )
        ])

    def test_wait(self):
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(self.env_assist.get_env(), wait=1),
            [
                fixture.error(
                    report_codes.WAIT_FOR_IDLE_NOT_LIVE_CLUSTER,
                ),
            ],
            expected_in_processor=False
        )


class WithWait(TestCase):
    def setUp(self):
        self.wait = 1
        self.env_assist, self.config = get_env_tools(self)
        (self.config
            .env.set_known_hosts_dests(KNOWN_HOSTS_DESTS)
            .runner.pcmk.can_wait()
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .http.host.check_auth(
                communication_list=[
                    dict(label=NODE_NAME, dest_list=NODE_DEST_LIST)
                ],
            )
            .local.get_host_info(NODE_NAME, NODE_DEST_LIST)
            .local.push_existing_authkey_to_remote(NODE_NAME, NODE_DEST_LIST)
            .local.run_pacemaker_remote(NODE_NAME, NODE_DEST_LIST)
            .local.push_cib(wait=self.wait)
        )

    def test_success_when_resource_started(self):
        (self.config
            .runner.pcmk.load_state(raw_resources=dict(
                resource_id=VIRTUAL_MACHINE_ID,
                resource_agent="ocf::pacemaker:remote",
                node_name=NODE_1,
            ))
            # self.config.fs is used to mock authkey file existence. Therefore,
            # filesystem mocking is active and we need to cover it. We tell the
            # file doesn't exist, because we aren't currently mocking the
            # function which reads it:
            #   if os.path.isfile(settings.crm_mon_schema):
            #       etree.RelaxNG(file=settings.crm_mon_schema).assertValid(dom)
            .fs.isfile(settings.crm_mon_schema, return_value=False)
        )
        node_add_guest(self.env_assist.get_env(), wait=self.wait)
        self.env_assist.assert_reports(
            REPORTS
                .info(
                    "resource_running",
                    report_codes.RESOURCE_RUNNING_ON_NODES,
                    roles_with_nodes={"Started": [NODE_1]},
                    resource_id=VIRTUAL_MACHINE_ID
                )
        )

    def test_fail_when_resource_not_started(self):
        (self.config
            .runner.pcmk.load_state(raw_resources=dict(
                resource_id=VIRTUAL_MACHINE_ID,
                resource_agent="ocf::pacemaker:remote",
                node_name=NODE_1,
                failed="true",
            ))
            # self.config.fs is used to mock authkey file existence. Therefore,
            # filesystem mocking is active and we need to cover it. We tell the
            # file doesn't exist, because we aren't currently mocking the
            # function which reads it:
            #   if os.path.isfile(settings.crm_mon_schema):
            #       etree.RelaxNG(file=settings.crm_mon_schema).assertValid(dom)
            .fs.isfile(settings.crm_mon_schema, return_value=False)
        )
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(self.env_assist.get_env(), wait=self.wait),
            [
                fixture.error(
                    report_codes.RESOURCE_DOES_NOT_RUN,
                    resource_id=VIRTUAL_MACHINE_ID,
                )
            ]
        )
        self.env_assist.assert_reports(REPORTS)


class RemoteService(TestCase):
    def setUp(self):
        self.env_assist, self.config = get_env_tools(self)
        (self.config
            .env.set_known_hosts_dests(KNOWN_HOSTS_DESTS)
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .http.host.check_auth(
                communication_list=[
                    dict(label=NODE_NAME, dest_list=NODE_DEST_LIST)
                ],
            )
            .local.get_host_info(NODE_NAME, NODE_DEST_LIST)
            .local.push_existing_authkey_to_remote(NODE_NAME, NODE_DEST_LIST)
        )

    def test_fails_when_offline(self):
        (self.config
            .local.run_pacemaker_remote(
                NODE_NAME, NODE_DEST_LIST, **FAIL_HTTP_KWARGS
            )
        )
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(self.env_assist.get_env()),
        )

        self.env_assist.assert_reports(
            REPORTS[:"pcmk_remote_enable_success"]
            +
            EXTRA_REPORTS.select("manage_services_connection_failed")
        )

    def test_fail_when_remotely_fail(self):
        (self.config
            .local.run_pacemaker_remote(NODE_NAME, NODE_DEST_LIST, result={
                "code": "fail",
                "message": "Action failed",
            })
        )
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(self.env_assist.get_env()),
        )
        self.env_assist.assert_reports(
            REPORTS[:"pcmk_remote_enable_success"] + EXTRA_REPORTS.select(
                "pcmk_remote_enable_failed",
                "pcmk_remote_start_failed",
            )
        )

    def test_forceable_when_remotely_fail(self):
        (self.config
            .local.run_pacemaker_remote(NODE_NAME, NODE_DEST_LIST, result={
                "code": "fail",
                "message": "Action failed",
            })
            .local.push_cib()
        )
        node_add_guest(
            self.env_assist.get_env(),
            allow_pacemaker_remote_service_fail=True
        )

        self.env_assist.assert_reports(
            REPORTS[:"pcmk_remote_enable_success"] + EXTRA_REPORTS.select(
                "pcmk_remote_enable_failed_warn",
                "pcmk_remote_start_failed_warn",
            )
        )


class AuthkeyDistribution(TestCase):
    def setUp(self):
        self.env_assist, self.config = get_env_tools(self)
        (self.config
            .env.set_known_hosts_dests(KNOWN_HOSTS_DESTS)
            .local.load_cib()
            .corosync_conf.load(node_name_list=[NODE_1, NODE_2])
            .http.host.check_auth(
                communication_list=[
                    dict(label=NODE_NAME, dest_list=NODE_DEST_LIST)
                ],
            )
            .local.get_host_info(NODE_NAME, NODE_DEST_LIST)
        )

    def test_fails_when_offline(self):
        pcmk_authkey_content = b"password"
        (self.config
            .local.authkey_exists(return_value=True)
            .local.open_authkey(pcmk_authkey_content)
            .local.distribute_authkey(
                communication_list=[
                    dict(label=NODE_NAME, dest_list=NODE_DEST_LIST)
                ],
                pcmk_authkey_content=pcmk_authkey_content,
                **FAIL_HTTP_KWARGS
            )
        )
        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(self.env_assist.get_env())
        )
        self.env_assist.assert_reports(
            REPORTS[:"authkey_distribution_success"] + EXTRA_REPORTS.only(
                "manage_services_connection_failed",
                command="remote/put_file",
            )
        )

    def test_fail_when_remotely_fail(self):
        (self.config
            .local.push_existing_authkey_to_remote(
                NODE_NAME,
                NODE_DEST_LIST,
                distribution_result={
                    "code": "conflict",
                    "message": "",
                }
            )
        )

        self.env_assist.assert_raise_library_error(
            lambda: node_add_guest(self.env_assist.get_env())
        )

        self.env_assist.assert_reports(
            REPORTS[:"authkey_distribution_success"]
            +
            EXTRA_REPORTS.select("authkey_distribution_failed")
        )

    def test_forceable_when_remotely_fail(self):
        (self.config
            .local.push_existing_authkey_to_remote(
                NODE_NAME,
                NODE_DEST_LIST,
                distribution_result={
                    "code": "conflict",
                    "message": "",
                }
            )
            .local.run_pacemaker_remote(NODE_NAME, NODE_DEST_LIST)
            .local.push_cib()
        )

        node_add_guest(
            self.env_assist.get_env(),
            allow_incomplete_distribution=True,
        )

        self.env_assist.assert_reports(
            REPORTS.remove("authkey_distribution_success")
            +
            EXTRA_REPORTS.select("authkey_distribution_failed_warn")
        )



from unittest import TestCase
from lxml import etree

from pcs.lib.cib import status


class GetResourcesFailcounts(TestCase):
    def test_no_failures(self):
        status_xml = etree.fromstring("""
            <status>
                <node_state uname="node1">
                    <transient_attributes>
                        <instance_attributes>
                        </instance_attributes>
                    </transient_attributes>
                </node_state>
                <node_state uname="node2">
                    <transient_attributes>
                    </transient_attributes>
                </node_state>
                <node_state uname="node3">
                </node_state>
            </status>
        """)
        self.assertEqual(
            status.get_resources_failcounts(status_xml),
            []
        )

    def test_failures(self):
        status_xml = etree.fromstring("""
            <status>
                <node_state uname="node1">
                    <transient_attributes>
                        <instance_attributes>
                            <nvpair name="fail-count-clone:0#start_0"
                                value="INFINITY"/>
                            <nvpair name="last-failure-clone:0#start_0"
                                value="1528871936"/>
                            <nvpair name="fail-count-clone:1#start_0"
                                value="999"/>
                            <nvpair name="last-failure-clone:1#start_0"
                                value="1528871937"/>
                        </instance_attributes>
                    </transient_attributes>
                </node_state>
                <node_state uname="node2">
                    <transient_attributes>
                        <instance_attributes>
                            <nvpair name="fail-count-resource#monitor_500"
                                value="10"/>
                            <nvpair name="last-failure-resource#monitor_500"
                                value="1528871946"/>
                            <nvpair name="fail-count-no-last#stop_0"
                                value="3"/>
                            <nvpair name="last-failure-no-count#monitor_1000"
                                value="1528871956"/>
                            <nvpair name="ignored-resource#monitor_1000"
                                value="ignored"/>
                            <nvpair name="fail-count-no-int#start_0"
                                value="a few"/>
                            <nvpair name="last-failure-no-int#start_0"
                                value="an hour ago"/>
                        </instance_attributes>
                    </transient_attributes>
                </node_state>
            </status>
        """)
        self.assertEqual(
            sorted(
                status.get_resources_failcounts(status_xml),
                key=lambda x: [str(x[key]) for key in sorted(x.keys())]
            ),
            sorted([
                {
                    "node": "node1",
                    "resource": "clone",
                    "clone_id": "0",
                    "operation": "start",
                    "interval": "0",
                    "fail_count": "INFINITY",
                    "last_failure": 1528871936,
                },
                {
                    "node": "node1",
                    "resource": "clone",
                    "clone_id": "1",
                    "operation": "start",
                    "interval": "0",
                    "fail_count": 999,
                    "last_failure": 1528871937,
                },
                {
                    "node": "node2",
                    "resource": "resource",
                    "clone_id": None,
                    "operation": "monitor",
                    "interval": "500",
                    "fail_count": 10,
                    "last_failure": 1528871946,
                },
                {
                    "node": "node2",
                    "resource": "no-last",
                    "clone_id": None,
                    "operation": "stop",
                    "interval": "0",
                    "fail_count": 3,
                    "last_failure": 0,
                },
                {
                    "node": "node2",
                    "resource": "no-count",
                    "clone_id": None,
                    "operation": "monitor",
                    "interval": "1000",
                    "fail_count": 0,
                    "last_failure": 1528871956,
                },
                {
                    "node": "node2",
                    "resource": "no-int",
                    "clone_id": None,
                    "operation": "start",
                    "interval": "0",
                    "fail_count": 1,
                    "last_failure": 0,
                },
            ],
                key=lambda x: [str(x[key]) for key in sorted(x.keys())]
            )
        )

class ParseFailureName(TestCase):
    # pylint: disable=protected-access
    def test_without_clone_id(self):
        self.assertEqual(
            status._parse_failure_name("resource#monitor_1000"),
            ("resource", None, "monitor", "1000")
        )

    def test_with_clone_id(self):
        self.assertEqual(
            status._parse_failure_name("resource:2#monitor_1000"),
            ("resource", "2", "monitor", "1000")
        )

class FilterResourceFailcounts(TestCase):
    # pylint: disable=too-many-instance-attributes
    def setUp(self):
        self.fail_01 = {
            "node": "nodeA",
            "resource": "resourceA",
            "clone_id": None,
            "operation": "start",
            "interval": "0",
            "fail_count": "INFINITY",
            "last_failure": "100",
        }
        self.fail_02 = {
            "node": "nodeA",
            "resource": "resourceB",
            "clone_id": None,
            "operation": "monitor",
            "interval": "1000",
            "fail_count": "INFINITY",
            "last_failure": "100",
        }
        self.fail_03 = {
            "node": "nodeB",
            "resource": "resourceA",
            "clone_id": None,
            "operation": "monitor",
            "interval": "1000",
            "fail_count": "INFINITY",
            "last_failure": "100",
        }
        self.fail_04 = {
            "node": "nodeB",
            "resource": "resourceB",
            "clone_id": None,
            "operation": "start",
            "interval": "0",
            "fail_count": "INFINITY",
            "last_failure": "100",
        }
        self.fail_05 = {
            "node": "nodeB",
            "resource": "resourceA",
            "clone_id": None,
            "operation": "monitor",
            "interval": "500",
            "fail_count": "INFINITY",
            "last_failure": "100",
        }
        self.fail_06 = {
            "node": "nodeB",
            "resource": "resourceA",
            "clone_id": None,
            "operation": "start",
            "interval": "0",
            "fail_count": "INFINITY",
            "last_failure": "100",
        }
        self.fail_07 = {
            "node": "nodeB",
            "resource": "resourceB",
            "clone_id": None,
            "operation": "monitor",
            "interval": "1000",
            "fail_count": "INFINITY",
            "last_failure": "100",
        }
        self.failures = [
            self.fail_01, self.fail_02, self.fail_03, self.fail_04,
            self.fail_05, self.fail_06, self.fail_07
        ]

    def test_no_filter(self):
        self.assertEqual(
            status.filter_resources_failcounts(self.failures),
            self.failures
        )

    def test_no_match(self):
        self.assertEqual(
            status.filter_resources_failcounts(
                self.failures, resource="resourceX"
            ),
            []
        )

    def test_filter_by_resource(self):
        self.assertEqual(
            status.filter_resources_failcounts(
                self.failures, resource="resourceA"
            ),
            [self.fail_01, self.fail_03, self.fail_05, self.fail_06]
        )

    def test_filter_by_node(self):
        self.assertEqual(
            status.filter_resources_failcounts(
                self.failures, node="nodeA"
            ),
            [self.fail_01, self.fail_02]
        )

    def test_filter_by_operation(self):
        self.assertEqual(
            status.filter_resources_failcounts(
                self.failures, operation="monitor"
            ),
            [self.fail_02, self.fail_03, self.fail_05, self.fail_07]
        )

    def test_filter_by_operation_and_interval(self):
        self.assertEqual(
            status.filter_resources_failcounts(
                self.failures, operation="monitor", interval="500"
            ),
            [self.fail_05]
        )

    def test_filter_by_operation_and_interval_int(self):
        self.assertEqual(
            status.filter_resources_failcounts(
                self.failures, operation="monitor", interval=500
            ),
            [self.fail_05]
        )

    def test_filter_by_resource_and_node(self):
        self.assertEqual(
            status.filter_resources_failcounts(
                self.failures, resource="resourceA", node="nodeB"
            ),
            [self.fail_03, self.fail_05, self.fail_06]
        )

    def test_filter_by_resource_and_node_and_operation(self):
        self.assertEqual(
            status.filter_resources_failcounts(
                self.failures, resource="resourceA", node="nodeB",
                operation="monitor"
            ),
            [self.fail_03, self.fail_05]
        )

    def test_filter_by_resource_and_node_and_operation_and_interval(self):
        self.assertEqual(
            status.filter_resources_failcounts(
                self.failures, resource="resourceA", node="nodeB",
                operation="monitor", interval="1000"
            ),
            [self.fail_03]
        )

    def test_filter_by_node_and_operation(self):
        self.assertEqual(
            status.filter_resources_failcounts(
                self.failures, node="nodeB", operation="monitor"
            ),
            [self.fail_03, self.fail_05, self.fail_07]
        )

    def test_filter_by_node_and_operation_and_interval(self):
        self.assertEqual(
            status.filter_resources_failcounts(
                self.failures,
                node="nodeB",
                operation="monitor",
                interval="1000",
            ),
            [self.fail_03, self.fail_07]
        )



import json

from pcs.common.node_communicator import RequestData
from pcs.lib import reports
from pcs.lib.communication.tools import (
    AllAtOnceStrategyMixin,
    AllSameDataMixin,
    OneByOneStrategyMixin,
    RunRemotelyBase,
    SkipOfflineMixin,
)
from pcs.lib.errors import ReportItemSeverity
from pcs.lib.node_communication import response_to_report_item


class CheckCorosyncOffline(
    SkipOfflineMixin, AllSameDataMixin, AllAtOnceStrategyMixin, RunRemotelyBase
):
    def __init__(
        self, report_processor,
        skip_offline_targets=False, allow_skip_offline=True,
    ):
        super(CheckCorosyncOffline, self).__init__(report_processor)
        if allow_skip_offline:
            self._set_skip_offline(skip_offline_targets)

    def _get_request_data(self):
        return RequestData("remote/status")

    def _process_response(self, response):
        report = self._get_response_report(response)
        node_label = response.request.target.label
        if report is not None:
            self._report_list([
                report,
                reports.corosync_not_running_check_node_error(
                    node_label,
                    self._failure_severity,
                    self._failure_forceable,
                )
            ])
            return
        try:
            status = response.data
            if not json.loads(status)["corosync"]:
                report = reports.corosync_not_running_on_node_ok(node_label)
            else:
                report = reports.corosync_running_on_node_fail(node_label)
        except (KeyError, json.JSONDecodeError):
            report = reports.corosync_not_running_check_node_error(
                node_label, self._failure_severity, self._failure_forceable
            )
        self._report(report)

    def before(self):
        self._report(reports.corosync_not_running_check_started())


class DistributeCorosyncConf(
    SkipOfflineMixin, AllSameDataMixin, AllAtOnceStrategyMixin, RunRemotelyBase
):
    def __init__(
        self, report_processor, config_text, skip_offline_targets=False,
        allow_skip_offline=True,
    ):
        super(DistributeCorosyncConf, self).__init__(report_processor)
        self._config_text = config_text
        if allow_skip_offline:
            self._set_skip_offline(skip_offline_targets)

    def _get_request_data(self):
        return RequestData(
            "remote/set_corosync_conf", [("corosync_conf", self._config_text)]
        )

    def _process_response(self, response):
        report = self._get_response_report(response)
        node_label = response.request.target.label
        if report is None:
            self._report(reports.corosync_config_accepted_by_node(node_label))
        else:
            self._report_list([
                report,
                reports.corosync_config_distribution_node_error(
                    node_label,
                    self._failure_severity,
                    self._failure_forceable,
                )
            ])

    def before(self):
        self._report(reports.corosync_config_distribution_started())


class ReloadCorosyncConf(
    AllSameDataMixin, OneByOneStrategyMixin, RunRemotelyBase
):
    __was_successful = False
    __has_failures = False

    def _get_request_data(self):
        return RequestData("remote/reload_corosync_conf")

    def _process_response(self, response):
        report = response_to_report_item(
            response, severity=ReportItemSeverity.WARNING
        )
        node = response.request.target.label
        if report is not None:
            self.__has_failures = True
            self._report(report)
            return self._get_next_list()
        try:
            output = json.loads(response.data)
            if output["code"] == "reloaded":
                self.__was_successful = True
                self._report(reports.corosync_config_reloaded(node))
                return []

            if output["code"] == "not_running":
                self._report(reports.corosync_config_reload_not_possible(node))
            else:
                self.__has_failures = True
                self._report(reports.corosync_config_reload_error(
                    output["message"],
                    node=node,
                    severity=ReportItemSeverity.WARNING,
                ))
        except (ValueError, LookupError):
            self.__has_failures = True
            self._report(reports.invalid_response_format(
                node,
                severity=ReportItemSeverity.WARNING,
            ))

        return self._get_next_list()

    def on_complete(self):
        if not self.__was_successful and self.__has_failures:
            self._report(reports.unable_to_perform_operation_on_any_node())



import shutil
from unittest import TestCase
from lxml import etree

from pcs_test.tools.cib import get_assert_pcs_effect_mixin
from pcs_test.tools.misc import get_test_resource as rc
from pcs_test.tools.pcs_runner import PcsRunner


class ManageUnmanage(
    TestCase,
    get_assert_pcs_effect_mixin(
        lambda cib: etree.tostring(
            # pylint:disable=undefined-variable
            etree.parse(cib).findall(".//resources")[0]
        )
    )
):
    empty_cib = rc("cib-empty.xml")
    temp_cib = rc("temp-cib.xml")

    @staticmethod
    def fixture_cib_unmanaged_a(add_empty_meta_b=False):
        empty_meta_b = '<meta_attributes id="B-meta_attributes" />'
        return """
            <resources>
                <primitive class="ocf" id="A" provider="heartbeat" type="Dummy">
                    <meta_attributes id="A-meta_attributes">
                        <nvpair id="A-meta_attributes-is-managed"
                            name="is-managed" value="false"
                        />
                    </meta_attributes>
                    <operations>
                        <op id="A-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
                <primitive class="ocf" id="B" provider="heartbeat" type="Dummy">
                    {empty_meta_b}<operations>
                        <op id="B-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>
        """.format(empty_meta_b=(empty_meta_b if add_empty_meta_b else ""))

    def setUp(self):
        shutil.copy(self.empty_cib, self.temp_cib)
        self.pcs_runner = PcsRunner(self.temp_cib)

    def fixture_resource(self, name, managed=True, with_monitors=False):
        self.assert_pcs_success(
            "resource create {0} ocf:heartbeat:Dummy --no-default-ops".format(
                name
            )
        )
        if not managed:
            self.assert_pcs_success(
                "resource unmanage {0} {1}".format(
                    name,
                    "--monitor" if with_monitors else ""
                )
            )

    def test_unmanage_none(self):
        self.assert_pcs_fail(
            "resource unmanage",
            "Error: You must specify resource(s) to unmanage\n"
        )

    def test_manage_none(self):
        self.assert_pcs_fail(
            "resource manage",
            "Error: You must specify resource(s) to manage\n"
        )

    def test_unmanage_one(self):
        self.fixture_resource("A")
        self.fixture_resource("B")
        self.assert_effect(
            "resource unmanage A",
            self.fixture_cib_unmanaged_a()
        )

    def test_manage_one(self):
        self.fixture_resource("A", managed=False)
        self.fixture_resource("B", managed=False)
        self.assert_effect(
            "resource manage B",
            self.fixture_cib_unmanaged_a(add_empty_meta_b=True)
        )

    def test_unmanage_monitor(self):
        self.fixture_resource("A")
        self.assert_effect(
            "resource unmanage A --monitor",
            """
            <resources>
                <primitive class="ocf" id="A" provider="heartbeat" type="Dummy">
                    <meta_attributes id="A-meta_attributes">
                        <nvpair id="A-meta_attributes-is-managed"
                            name="is-managed" value="false"
                        />
                    </meta_attributes>
                    <operations>
                        <op id="A-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s" enabled="false"
                        />
                    </operations>
                </primitive>
            </resources>
            """
        )

    def test_unmanage_monitor_enabled(self):
        self.fixture_resource("A")
        self.assert_effect(
            "resource unmanage A",
            """
            <resources>
                <primitive class="ocf" id="A" provider="heartbeat" type="Dummy">
                    <meta_attributes id="A-meta_attributes">
                        <nvpair id="A-meta_attributes-is-managed"
                            name="is-managed" value="false"
                        />
                    </meta_attributes>
                    <operations>
                        <op id="A-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>
            """
        )

    def test_manage_monitor(self):
        self.fixture_resource("A", managed=True, with_monitors=True)
        self.assert_effect(
            "resource manage A --monitor",
            """
            <resources>
                <primitive class="ocf" id="A" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="A-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>
            """
        )

    def test_manage_monitor_disabled(self):
        self.fixture_resource("A", managed=False, with_monitors=True)
        self.assert_effect(
            "resource manage A",
            """
            <resources>
                <primitive class="ocf" id="A" provider="heartbeat" type="Dummy">
                    <meta_attributes id="A-meta_attributes" />
                    <operations>
                        <op id="A-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s" enabled="false"
                        />
                    </operations>
                </primitive>
            </resources>
            """,
            "Warning: Resource 'A' has no enabled monitor operations."
                " Re-run with '--monitor' to enable them.\n"
        )

    def test_unmanage_more(self):
        self.fixture_resource("A")
        self.fixture_resource("B")
        self.assert_effect(
            "resource unmanage A B",
            """
            <resources>
                <primitive class="ocf" id="A" provider="heartbeat" type="Dummy">
                    <meta_attributes id="A-meta_attributes">
                        <nvpair id="A-meta_attributes-is-managed"
                            name="is-managed" value="false"
                        />
                    </meta_attributes>
                    <operations>
                        <op id="A-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
                <primitive class="ocf" id="B" provider="heartbeat" type="Dummy">
                    <meta_attributes id="B-meta_attributes">
                        <nvpair id="B-meta_attributes-is-managed"
                            name="is-managed" value="false"
                        />
                    </meta_attributes>
                    <operations>
                        <op id="B-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>
            """
        )

    def test_manage_more(self):
        self.fixture_resource("A", managed=False)
        self.fixture_resource("B", managed=False)
        self.assert_effect(
            "resource manage A B",
            """
            <resources>
                <primitive class="ocf" id="A" provider="heartbeat" type="Dummy">
                    <meta_attributes id="A-meta_attributes" />
                    <operations>
                        <op id="A-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
                <primitive class="ocf" id="B" provider="heartbeat" type="Dummy">
                    <meta_attributes id="B-meta_attributes" />
                    <operations>
                        <op id="B-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>
            """
        )

    def test_unmanage_nonexistent(self):
        self.fixture_resource("A")

        self.assert_pcs_fail(
            "resource unmanage A B",
            "Error: bundle/clone/group/resource 'B' does not exist\n"
        )
        self.assert_resources_xml_in_cib(
            """
            <resources>
                <primitive class="ocf" id="A" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="A-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>
            """
        )

    def test_manage_nonexistent(self):
        self.fixture_resource("A", managed=False)

        self.assert_pcs_fail(
            "resource manage A B",
            "Error: bundle/clone/group/resource 'B' does not exist\n"
        )
        self.assert_resources_xml_in_cib(
            """
            <resources>
                <primitive class="ocf" id="A" provider="heartbeat" type="Dummy">
                    <meta_attributes id="A-meta_attributes">
                        <nvpair id="A-meta_attributes-is-managed"
                            name="is-managed" value="false"
                        />
                    </meta_attributes>
                    <operations>
                        <op id="A-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>
            """
        )



from tornado.httpserver import HTTPServer
from tornado.netutil import bind_sockets

from pcs.daemon.ssl import PcsdSSL
from pcs.daemon import log

class HttpsServerManageException(Exception):
    pass

class HttpsServerManage:
    """
    Instance of HttpsServerManage encapsulates the construction of an HTTPServer
    """

    # Main motivation for this object is to be able to change the ssl
    # certificates from an http request without restarting the whole pcsd
    # daemon.
    #
    # For this purpose an application, which handles http requests, gets
    # a reference to the HttpsServerManage instance. When new certificates
    # arrive via a request the application asks the HttpsServerManage instance
    # for necessary steps (it stops listening of the current HTTPServer and
    # starts a new one with updated certificates).

    def __init__(self, make_app, port, bind_addresses, ssl: PcsdSSL):
        self.__make_app = make_app
        self.__port = port
        self.__bind_addresses = bind_addresses

        self.__server = None
        self.__ssl = ssl
        self.__server_is_running = False

    @property
    def server_is_running(self):
        return self.__server_is_running

    def stop(self):
        self.__server.stop()
        self.__server_is_running = False

    def start(self):
        self.__ssl.guarantee_valid_certs()

        log.pcsd.info("Starting server...")

        self.__server = HTTPServer(
            self.__make_app(self),
            ssl_options=self.__ssl.create_context()
        )

        # It is necessary to bind sockets for every new HTTPServer since
        # HTTPServer.stop calls sock.close() inside.
        sockets = []
        for address in self.__bind_addresses:
            log.pcsd.info(
                "Binding socket for address '%s' and port '%s'",
                address if address is not None else "*",
                self.__port
            )
            sockets.extend(bind_sockets(self.__port, address))

        self.__server.add_sockets(sockets)

        log.pcsd.info("Server is listening")
        self.__server_is_running = True
        return self

    def reload_certs(self):
        if not self.server_is_running:
            raise HttpsServerManageException(
                "Could not reload certificates, server is not running"
            )
        log.pcsd.info("Stopping server to reload ssl certificates...")
        self.stop()
        self.start()



# pylint: disable=line-too-long

# force categories
FORCE = "FORCE"
FORCE_ALERT_RECIPIENT_VALUE_NOT_UNIQUE = "FORCE_ALERT_RECIPIENT_VALUE_NOT_UNIQUE"
FORCE_ALREADY_IN_CLUSTER = "FORCE_ALREADY_IN_CLUSTER"
FORCE_BOOTH_DESTROY = "FORCE_BOOTH_DESTROY"
FORCE_BOOTH_REMOVE_FROM_CIB = "FORCE_BOOTH_REMOVE_FROM_CIB"
FORCE_REMOVE_MULTIPLE_NODES = "FORCE_REMOVE_MULTIPLE_NODES"
FORCE_CONSTRAINT_DUPLICATE = "FORCE_CONSTRAINT_DUPLICATE"
FORCE_CONSTRAINT_MULTIINSTANCE_RESOURCE = "FORCE_CONSTRAINT_MULTIINSTANCE_RESOURCE"
FORCE_FILE_OVERWRITE = "FORCE_FILE_OVERWRITE"
FORCE_LOAD_NODES_FROM_CIB = "FORCE_LOAD_NODES_FROM_CIB"
FORCE_LOAD_THRESHOLD = "FORCE_LOAD_THRESHOLD"
FORCE_METADATA_ISSUE = "FORCE_METADATA_ISSUE"
FORCE_NODE_ADDRESSES_UNRESOLVABLE = "FORCE_NODE_ADDRESSES_UNRESOLVABLE"
FORCE_NODE_DOES_NOT_EXIST = "FORCE_NODE_DOES_NOT_EXIST"
FORCE_OPTIONS = "FORCE_OPTIONS"
FORCE_QDEVICE_MODEL = "FORCE_QDEVICE_MODEL"
FORCE_QDEVICE_USED = "FORCE_QDEVICE_USED"
FORCE_QUORUM_LOSS = "FORCE_QUORUM_LOSS"
FORCE_STONITH_RESOURCE_DOES_NOT_EXIST = "FORCE_STONITH_RESOURCE_DOES_NOT_EXIST"
FORCE_NOT_SUITABLE_COMMAND = "FORCE_NOT_SUITABLE_COMMAND"
FORCE_CLEAR_CLUSTER_NODE = "FORCE_CLEAR_CLUSTER_NODE"
FORCE_RESOURCE_IN_BUNDLE_NOT_ACCESSIBLE = "FORCE_RESOURCE_IN_BUNDLE_NOT_ACCESSIBLE"
SKIP_OFFLINE_NODES = "SKIP_OFFLINE_NODES"
SKIP_FILE_DISTRIBUTION_ERRORS = "SKIP_FILE_DISTRIBUTION_ERRORS"
SKIP_ACTION_ON_NODES_ERRORS = "SKIP_ACTION_ON_NODES_ERRORS"
SKIP_UNREADABLE_CONFIG = "SKIP_UNREADABLE_CONFIG"

AGENT_NAME_GUESS_FOUND_MORE_THAN_ONE = "AGENT_NAME_GUESS_FOUND_MORE_THAN_ONE"
AGENT_NAME_GUESS_FOUND_NONE = "AGENT_NAME_GUESS_FOUND_NONE"
AGENT_NAME_GUESSED = "AGENT_NAME_GUESSED"
BAD_CLUSTER_STATE_FORMAT = 'BAD_CLUSTER_STATE_FORMAT'
BOOTH_ADDRESS_DUPLICATION = "BOOTH_ADDRESS_DUPLICATION"
BOOTH_ALREADY_IN_CIB = "BOOTH_ALREADY_IN_CIB"
BOOTH_CANNOT_DETERMINE_LOCAL_SITE_IP = "BOOTH_CANNOT_DETERMINE_LOCAL_SITE_IP"
BOOTH_CONFIG_ACCEPTED_BY_NODE = "BOOTH_CONFIG_ACCEPTED_BY_NODE"
BOOTH_CONFIG_DISTRIBUTION_NODE_ERROR = "BOOTH_CONFIG_DISTRIBUTION_NODE_ERROR"
BOOTH_CONFIG_DISTRIBUTION_STARTED = "BOOTH_CONFIG_DISTRIBUTION_STARTED"
BOOTH_CONFIG_IS_USED = "BOOTH_CONFIG_IS_USED"
BOOTH_CONFIG_UNEXPECTED_LINES = "BOOTH_CONFIG_UNEXPECTED_LINES"
BOOTH_DAEMON_STATUS_ERROR = "BOOTH_DAEMON_STATUS_ERROR"
BOOTH_EVEN_PEERS_NUM = "BOOTH_EVEN_PEERS_NUM"
BOOTH_FETCHING_CONFIG_FROM_NODE = "BOOTH_FETCHING_CONFIG_FROM_NODE"
BOOTH_INVALID_NAME = "BOOTH_INVALID_NAME"
BOOTH_LACK_OF_SITES = "BOOTH_LACK_OF_SITES"
BOOTH_MULTIPLE_TIMES_IN_CIB = "BOOTH_MULTIPLE_TIMES_IN_CIB"
BOOTH_NOT_EXISTS_IN_CIB = "BOOTH_NOT_EXISTS_IN_CIB"
BOOTH_PEERS_STATUS_ERROR = "BOOTH_PEERS_STATUS_ERROR"
BOOTH_TICKET_DOES_NOT_EXIST = "BOOTH_TICKET_DOES_NOT_EXIST"
BOOTH_TICKET_DUPLICATE = "BOOTH_TICKET_DUPLICATE"
BOOTH_TICKET_NAME_INVALID = "BOOTH_TICKET_NAME_INVALID"
BOOTH_TICKET_OPERATION_FAILED = "BOOTH_TICKET_OPERATION_FAILED"
BOOTH_TICKET_STATUS_ERROR = "BOOTH_TICKET_STATUS_ERROR"
BOOTH_UNSUPPORTED_FILE_LOCATION = "BOOTH_UNSUPPORTED_FILE_LOCATION"
CANNOT_BAN_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE = "CANNOT_BAN_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE"
CANNOT_BAN_RESOURCE_STOPPED_NO_NODE_SPECIFIED = "CANNOT_BAN_RESOURCE_STOPPED_NO_NODE_SPECIFIED"
CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_FOR_NEW_GROUP = "CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_FOR_NEW_GROUP"
CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_NOT_IN_GROUP = "CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_NOT_IN_GROUP"
CANNOT_GROUP_RESOURCE_ALREADY_IN_THE_GROUP = "CANNOT_GROUP_RESOURCE_ALREADY_IN_THE_GROUP"
CANNOT_GROUP_RESOURCE_MORE_THAN_ONCE = "CANNOT_GROUP_RESOURCE_MORE_THAN_ONCE"
CANNOT_GROUP_RESOURCE_NEXT_TO_ITSELF = "CANNOT_GROUP_RESOURCE_NEXT_TO_ITSELF"
CANNOT_GROUP_RESOURCE_NO_RESOURCES = "CANNOT_GROUP_RESOURCE_NO_RESOURCES"
CANNOT_GROUP_RESOURCE_WRONG_TYPE = "CANNOT_GROUP_RESOURCE_WRONG_TYPE"
CANNOT_MOVE_RESOURCE_BUNDLE = "CANNOT_MOVE_RESOURCE_BUNDLE"
CANNOT_MOVE_RESOURCE_CLONE = "CANNOT_MOVE_RESOURCE_CLONE"
CANNOT_MOVE_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE = "CANNOT_MOVE_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE"
CANNOT_MOVE_RESOURCE_PROMOTABLE_NOT_MASTER = "CANNOT_MOVE_RESOURCE_PROMOTABLE_NOT_MASTER"
CANNOT_MOVE_RESOURCE_STOPPED_NO_NODE_SPECIFIED = "CANNOT_MOVE_RESOURCE_STOPPED_NO_NODE_SPECIFIED"
CANNOT_REMOVE_ALL_CLUSTER_NODES = "CANNOT_REMOVE_ALL_CLUSTER_NODES"
CANNOT_UNMOVE_UNBAN_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE = "CANNOT_UNMOVE_UNBAN_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE"
CIB_ACL_ROLE_IS_ALREADY_ASSIGNED_TO_TARGET = "CIB_ACL_ROLE_IS_ALREADY_ASSIGNED_TO_TARGET"
CIB_ACL_ROLE_IS_NOT_ASSIGNED_TO_TARGET = "CIB_ACL_ROLE_IS_NOT_ASSIGNED_TO_TARGET"
CIB_ACL_TARGET_ALREADY_EXISTS = "CIB_ACL_TARGET_ALREADY_EXISTS"
CIB_ALERT_RECIPIENT_ALREADY_EXISTS = "CIB_ALERT_RECIPIENT_ALREADY_EXISTS"
CIB_ALERT_RECIPIENT_VALUE_INVALID = "CIB_ALERT_RECIPIENT_VALUE_INVALID"
CIB_CANNOT_FIND_MANDATORY_SECTION = "CIB_CANNOT_FIND_MANDATORY_SECTION"
CIB_DIFF_ERROR = "CIB_DIFF_ERROR"
CIB_FENCING_LEVEL_ALREADY_EXISTS = "CIB_FENCING_LEVEL_ALREADY_EXISTS"
CIB_FENCING_LEVEL_DOES_NOT_EXIST = "CIB_FENCING_LEVEL_DOES_NOT_EXIST"
CIB_LOAD_ERROR_BAD_FORMAT = "CIB_LOAD_ERROR_BAD_FORMAT"
CIB_LOAD_ERROR = "CIB_LOAD_ERROR"
CIB_LOAD_ERROR_GET_NODES_FOR_VALIDATION = "CIB_LOAD_ERROR_GET_NODES_FOR_VALIDATION"
CIB_LOAD_ERROR_SCOPE_MISSING = "CIB_LOAD_ERROR_SCOPE_MISSING"
CIB_PUSH_FORCED_FULL_DUE_TO_CRM_FEATURE_SET = "CIB_PUSH_FORCED_FULL_DUE_TO_CRM_FEATURE_SET"
CIB_PUSH_ERROR = "CIB_PUSH_ERROR"
CIB_SAVE_TMP_ERROR = "CIB_SAVE_TMP_ERROR"
CIB_SIMULATE_ERROR = "CIB_SIMULATE_ERROR"
CIB_UPGRADE_FAILED = "CIB_UPGRADE_FAILED"
CIB_UPGRADE_FAILED_TO_MINIMAL_REQUIRED_VERSION = "CIB_UPGRADE_FAILED_TO_MINIMAL_REQUIRED_VERSION"
CIB_UPGRADE_SUCCESSFUL = "CIB_UPGRADE_SUCCESSFUL"
CLUSTER_DESTROY_STARTED = "CLUSTER_DESTROY_STARTED"
CLUSTER_DESTROY_SUCCESS = "CLUSTER_DESTROY_SUCCESS"
CLUSTER_ENABLE_STARTED = "CLUSTER_ENABLE_STARTED"
CLUSTER_ENABLE_SUCCESS = "CLUSTER_ENABLE_SUCCESS"
CLUSTER_RESTART_REQUIRED_TO_APPLY_CHANGES = "CLUSTER_RESTART_REQUIRED_TO_APPLY_CHANGES"
CLUSTER_SETUP_SUCCESS = "CLUSTER_SETUP_SUCCESS"
CLUSTER_START_STARTED = "CLUSTER_START_STARTED"
CLUSTER_START_SUCCESS = "CLUSTER_START_SUCCESS"
CLUSTER_WILL_BE_DESTROYED = "CLUSTER_WILL_BE_DESTROYED"
LIVE_ENVIRONMENT_NOT_CONSISTENT = "LIVE_ENVIRONMENT_NOT_CONSISTENT"
LIVE_ENVIRONMENT_REQUIRED = "LIVE_ENVIRONMENT_REQUIRED"
LIVE_ENVIRONMENT_REQUIRED_FOR_LOCAL_NODE = "LIVE_ENVIRONMENT_REQUIRED_FOR_LOCAL_NODE"
COROSYNC_ADDRESS_IP_VERSION_WRONG_FOR_LINK = "COROSYNC_ADDRESS_IP_VERSION_WRONG_FOR_LINK"
COROSYNC_BAD_NODE_ADDRESSES_COUNT = "COROSYNC_BAD_NODE_ADDRESSES_COUNT"
COROSYNC_CONFIG_ACCEPTED_BY_NODE = "COROSYNC_CONFIG_ACCEPTED_BY_NODE"
COROSYNC_CONFIG_CANNOT_SAVE_INVALID_NAMES_VALUES = "COROSYNC_CONFIG_CANNOT_SAVE_INVALID_NAMES_VALUES"
COROSYNC_CONFIG_DISTRIBUTION_NODE_ERROR = "COROSYNC_CONFIG_DISTRIBUTION_NODE_ERROR"
COROSYNC_CONFIG_DISTRIBUTION_STARTED = "COROSYNC_CONFIG_DISTRIBUTION_STARTED"
COROSYNC_CONFIG_MISSING_NAMES_OF_NODES = "COROSYNC_CONFIG_MISSING_NAMES_OF_NODES"
COROSYNC_CONFIG_NO_NODES_DEFINED = "COROSYNC_CONFIG_NO_NODES_DEFINED"
COROSYNC_CONFIG_RELOADED = "COROSYNC_CONFIG_RELOADED"
COROSYNC_CONFIG_RELOAD_ERROR = "COROSYNC_CONFIG_RELOAD_ERROR"
COROSYNC_CONFIG_RELOAD_NOT_POSSIBLE = "COROSYNC_CONFIG_RELOAD_NOT_POSSIBLE"
COROSYNC_IP_VERSION_MISMATCH_IN_LINKS = "COROSYNC_IP_VERSION_MISMATCH_IN_LINKS"
COROSYNC_CANNOT_ADD_REMOVE_LINKS_BAD_TRANSPORT = "COROSYNC_CANNOT_ADD_REMOVE_LINKS_BAD_TRANSPORT"
COROSYNC_CANNOT_ADD_REMOVE_LINKS_NO_LINKS_SPECIFIED = "COROSYNC_CANNOT_ADD_REMOVE_LINKS_NO_LINKS_SPECIFIED"
COROSYNC_CANNOT_ADD_REMOVE_LINKS_TOO_MANY_FEW_LINKS = "COROSYNC_CANNOT_ADD_REMOVE_LINKS_TOO_MANY_FEW_LINKS"
COROSYNC_LINK_ALREADY_EXISTS_CANNOT_ADD = "COROSYNC_LINK_ALREADY_EXISTS_CANNOT_ADD"
COROSYNC_LINK_DOES_NOT_EXIST_CANNOT_REMOVE = "COROSYNC_LINK_DOES_NOT_EXIST_CANNOT_REMOVE"
COROSYNC_LINK_DOES_NOT_EXIST_CANNOT_UPDATE = "COROSYNC_LINK_DOES_NOT_EXIST_CANNOT_UPDATE"
COROSYNC_LINK_NUMBER_DUPLICATION = "COROSYNC_LINK_NUMBER_DUPLICATION"
COROSYNC_NODE_ADDRESS_COUNT_MISMATCH = "COROSYNC_NODE_ADDRESS_COUNT_MISMATCH"
COROSYNC_NODE_CONFLICT_CHECK_SKIPPED = "COROSYNC_NODE_CONFLICT_CHECK_SKIPPED"
COROSYNC_NODES_MISSING = "COROSYNC_NODES_MISSING"
COROSYNC_NOT_RUNNING_CHECK_NODE_ERROR = "COROSYNC_NOT_RUNNING_CHECK_NODE_ERROR"
COROSYNC_NOT_RUNNING_CHECK_STARTED = "COROSYNC_NOT_RUNNING_CHECK_STARTED"
COROSYNC_NOT_RUNNING_ON_NODE = "COROSYNC_NOT_RUNNING_ON_NODE"
COROSYNC_OPTIONS_INCOMPATIBLE_WITH_QDEVICE = "COROSYNC_OPTIONS_INCOMPATIBLE_WITH_QDEVICE"
COROSYNC_QUORUM_ATB_CANNOT_BE_DISABLED_DUE_TO_SBD = "COROSYNC_QUORUM_ATB_CANNOT_BE_DISABLED_DUE_TO_SBD"
COROSYNC_QUORUM_ATB_WILL_BE_ENABLED_DUE_TO_SBD = "COROSYNC_QUORUM_ATB_WILL_BE_ENABLED_DUE_TO_SBD"
COROSYNC_QUORUM_GET_STATUS_ERROR = "COROSYNC_QUORUM_GET_STATUS_ERROR"
COROSYNC_QUORUM_HEURISTICS_ENABLED_WITH_NO_EXEC = "COROSYNC_QUORUM_HEURISTICS_ENABLED_WITH_NO_EXEC"
COROSYNC_QUORUM_LOSS_UNABLE_TO_CHECK = "COROSYNC_QUORUM_LOSS_UNABLE_TO_CHECK"
COROSYNC_QUORUM_SET_EXPECTED_VOTES_ERROR = "COROSYNC_QUORUM_SET_EXPECTED_VOTES_ERROR"
COROSYNC_QUORUM_WILL_BE_LOST = "COROSYNC_QUORUM_WILL_BE_LOST"
COROSYNC_RUNNING_ON_NODE = "COROSYNC_RUNNING_ON_NODE"
COROSYNC_TOO_MANY_LINKS_OPTIONS = "COROSYNC_TOO_MANY_LINKS_OPTIONS"
COROSYNC_TRANSPORT_UNSUPPORTED_OPTIONS = "COROSYNC_TRANSPORT_UNSUPPORTED_OPTIONS"
CRM_MON_ERROR = "CRM_MON_ERROR"
DEFAULTS_CAN_BE_OVERRIDEN = "DEFAULTS_CAN_BE_OVERRIDEN"
DEPRECATED_OPTION = "DEPRECATED_OPTION"
DUPLICATE_CONSTRAINTS_EXIST = "DUPLICATE_CONSTRAINTS_EXIST"
EMPTY_RESOURCE_SET_LIST = "EMPTY_RESOURCE_SET_LIST"
EMPTY_ID = "EMPTY_ID"
FENCE_HISTORY_COMMAND_ERROR = "FENCE_HISTORY_COMMAND_ERROR"
FENCE_HISTORY_NOT_SUPPORTED = "FENCE_HISTORY_NOT_SUPPORTED"
FILES_DISTRIBUTION_SKIPPED = "FILES_DISTRIBUTION_SKIPPED"
FILES_DISTRIBUTION_STARTED = "FILES_DISTRIBUTION_STARTED"
FILES_REMOVE_FROM_NODES_STARTED = "FILES_REMOVE_FROM_NODES_STARTED"
FILES_REMOVE_FROM_NODES_SKIPPED = "FILES_REMOVE_FROM_NODES_SKIPPED"
FILE_ALREADY_EXISTS = "FILE_ALREADY_EXISTS"
FILE_DISTRIBUTION_ERROR = "FILE_DISTRIBUTION_ERROR"
FILE_DISTRIBUTION_SUCCESS = "FILE_DISTRIBUTION_SUCCESS"
FILE_IO_ERROR = "FILE_IO_ERROR"
FILE_REMOVE_FROM_NODE_ERROR = "FILE_REMOVE_FROM_NODE_ERROR"
FILE_REMOVE_FROM_NODE_SUCCESS = "FILE_REMOVE_FROM_NODE_SUCCESS"
HOST_NOT_FOUND = "HOST_NOT_FOUND"
HOST_ALREADY_AUTHORIZED = "HOST_ALREADY_AUTHORIZED"
HOST_ALREADY_IN_CLUSTER_CONFIG = "HOST_ALREADY_IN_CLUSTER_CONFIG"
HOST_ALREADY_IN_CLUSTER_SERVICES = "HOST_ALREADY_IN_CLUSTER_SERVICES"
ID_ALREADY_EXISTS = 'ID_ALREADY_EXISTS'
ID_BELONGS_TO_UNEXPECTED_TYPE = "ID_BELONGS_TO_UNEXPECTED_TYPE"
ID_NOT_FOUND = 'ID_NOT_FOUND'
INVALID_CIB_CONTENT = "INVALID_CIB_CONTENT"
INVALID_ID = "INVALID_ID"
INVALID_OPTIONS = "INVALID_OPTIONS"
INVALID_USERDEFINED_OPTIONS = "INVALID_USERDEFINED_OPTIONS"
INVALID_OPTION_TYPE = "INVALID_OPTION_TYPE"
INVALID_OPTION_VALUE = "INVALID_OPTION_VALUE"
INVALID_RESOURCE_AGENT_NAME = 'INVALID_RESOURCE_AGENT_NAME'
INVALID_RESPONSE_FORMAT = "INVALID_RESPONSE_FORMAT"
INVALID_SCORE = "INVALID_SCORE"
INVALID_STONITH_AGENT_NAME = "INVALID_STONITH_AGENT_NAME"
INVALID_TIMEOUT_VALUE = "INVALID_TIMEOUT_VALUE"
MULTIPLE_SCORE_OPTIONS = "MULTIPLE_SCORE_OPTIONS"
MULTIPLE_RESULTS_FOUND = "MULTIPLE_RESULTS_FOUND"
MUTUALLY_EXCLUSIVE_OPTIONS = "MUTUALLY_EXCLUSIVE_OPTIONS"
NODE_ADDRESSES_ALREADY_EXIST = "NODE_ADDRESSES_ALREADY_EXIST"
NODE_ADDRESSES_CANNOT_BE_EMPTY = "NODE_ADDRESSES_CANNOT_BE_EMPTY"
NODE_ADDRESSES_DUPLICATION = "NODE_ADDRESSES_DUPLICATION"
NODE_ADDRESSES_UNRESOLVABLE = "NODE_ADDRESSES_UNRESOLVABLE"
NODE_COMMUNICATION_COMMAND_UNSUCCESSFUL = "NODE_COMMUNICATION_COMMAND_UNSUCCESSFUL"
NODE_COMMUNICATION_DEBUG_INFO = "NODE_COMMUNICATION_DEBUG_INFO"
NODE_COMMUNICATION_ERROR = "NODE_COMMUNICATION_ERROR"
NODE_COMMUNICATION_ERROR_NOT_AUTHORIZED = "NODE_COMMUNICATION_ERROR_NOT_AUTHORIZED"
NODE_COMMUNICATION_ERROR_PERMISSION_DENIED = "NODE_COMMUNICATION_ERROR_PERMISSION_DENIED"
NODE_COMMUNICATION_ERROR_UNABLE_TO_CONNECT = "NODE_COMMUNICATION_ERROR_UNABLE_TO_CONNECT"
NODE_COMMUNICATION_ERROR_UNSUPPORTED_COMMAND = "NODE_COMMUNICATION_ERROR_UNSUPPORTED_COMMAND"
NODE_COMMUNICATION_ERROR_TIMED_OUT = "NODE_COMMUNICATION_ERROR_TIMED_OUT"
NODE_COMMUNICATION_FINISHED = "NODE_COMMUNICATION_FINISHED"
NODE_COMMUNICATION_NOT_CONNECTED = "NODE_COMMUNICATION_NOT_CONNECTED"
NODE_COMMUNICATION_NO_MORE_ADDRESSES = "NODE_COMMUNICATION_NO_MORE_ADDRESSES"
NODE_COMMUNICATION_PROXY_IS_SET = "NODE_COMMUNICATION_PROXY_IS_SET"
NODE_COMMUNICATION_RETRYING = "NODE_COMMUNICATION_RETRYING"
NODE_COMMUNICATION_STARTED = "NODE_COMMUNICATION_STARTED"
NODE_NAMES_ALREADY_EXIST = "NODE_NAMES_ALREADY_EXIST"
NODE_NAMES_DUPLICATION = "NODE_NAMES_DUPLICATION"
NODE_NOT_FOUND = "NODE_NOT_FOUND"
NODE_REMOVE_IN_PACEMAKER_FAILED = "NODE_REMOVE_IN_PACEMAKER_FAILED"
NONE_HOST_FOUND = "NONE_HOST_FOUND"
NODE_USED_AS_TIE_BREAKER = "NODE_USED_AS_TIE_BREAKER"
NODES_TO_REMOVE_UNREACHABLE = "NODES_TO_REMOVE_UNREACHABLE"
NODE_TO_CLEAR_IS_STILL_IN_CLUSTER = "NODE_TO_CLEAR_IS_STILL_IN_CLUSTER"
OMITTING_NODE = "OMITTING_NODE"
OBJECT_WITH_ID_IN_UNEXPECTED_CONTEXT = "OBJECT_WITH_ID_IN_UNEXPECTED_CONTEXT"
PACEMAKER_LOCAL_NODE_NAME_NOT_FOUND = "PACEMAKER_LOCAL_NODE_NAME_NOT_FOUND"
PARSE_ERROR_COROSYNC_CONF = "PARSE_ERROR_COROSYNC_CONF"
PARSE_ERROR_COROSYNC_CONF_EXTRA_CHARACTERS_AFTER_OPENING_BRACE = "PARSE_ERROR_COROSYNC_CONF_EXTRA_CHARACTERS_AFTER_OPENING_BRACE"
PARSE_ERROR_COROSYNC_CONF_EXTRA_CHARACTERS_BEFORE_OR_AFTER_CLOSING_BRACE = "PARSE_ERROR_COROSYNC_CONF_EXTRA_CHARACTERS_BEFORE_OR_AFTER_CLOSING_BRACE"
PARSE_ERROR_COROSYNC_CONF_LINE_IS_NOT_SECTION_NOR_KEY_VALUE = "PARSE_ERROR_COROSYNC_CONF_LINE_IS_NOT_SECTION_NOR_KEY_VALUE"
PARSE_ERROR_COROSYNC_CONF_MISSING_CLOSING_BRACE = "PARSE_ERROR_COROSYNC_CONF_MISSING_CLOSING_BRACE"
PARSE_ERROR_COROSYNC_CONF_MISSING_SECTION_NAME_BEFORE_OPENING_BRACE = "PARSE_ERROR_COROSYNC_CONF_MISSING_SECTION_NAME_BEFORE_OPENING_BRACE"
PARSE_ERROR_COROSYNC_CONF_UNEXPECTED_CLOSING_BRACE = "PARSE_ERROR_COROSYNC_CONF_UNEXPECTED_CLOSING_BRACE"
PARSE_ERROR_JSON_FILE = "PARSE_ERROR_JSON_FILE"
PCSD_VERSION_TOO_OLD = "PCSD_VERSION_TOO_OLD"
PCSD_SSL_CERT_AND_KEY_DISTRIBUTION_STARTED = "PCSD_SSL_CERT_AND_KEY_DISTRIBUTION_STARTED"
PCSD_SSL_CERT_AND_KEY_SET_SUCCESS = "PCSD_SSL_CERT_AND_KEY_SET_SUCCESS"
PREREQUISITE_OPTION_MUST_BE_ENABLED_AS_WELL = "PREREQUISITE_OPTION_MUST_BE_ENABLED_AS_WELL"
PREREQUISITE_OPTION_MUST_BE_DISABLED = "PREREQUISITE_OPTION_MUST_BE_DISABLED"
PREREQUISITE_OPTION_MUST_NOT_BE_SET = "PREREQUISITE_OPTION_MUST_NOT_BE_SET"
PREREQUISITE_OPTION_IS_MISSING = "PREREQUISITE_OPTION_IS_MISSING"
QDEVICE_ALREADY_DEFINED = "QDEVICE_ALREADY_DEFINED"
QDEVICE_ALREADY_INITIALIZED = "QDEVICE_ALREADY_INITIALIZED"
QDEVICE_CERTIFICATE_ACCEPTED_BY_NODE = "QDEVICE_CERTIFICATE_ACCEPTED_BY_NODE"
QDEVICE_CERTIFICATE_DISTRIBUTION_STARTED = "QDEVICE_CERTIFICATE_DISTRIBUTION_STARTED"
QDEVICE_CERTIFICATE_REMOVAL_STARTED = "QDEVICE_CERTIFICATE_REMOVAL_STARTED"
QDEVICE_CERTIFICATE_REMOVED_FROM_NODE = "QDEVICE_CERTIFICATE_REMOVED_FROM_NODE"
QDEVICE_CERTIFICATE_IMPORT_ERROR = "QDEVICE_CERTIFICATE_IMPORT_ERROR"
QDEVICE_CERTIFICATE_SIGN_ERROR = "QDEVICE_CERTIFICATE_SIGN_ERROR"
QDEVICE_DESTROY_ERROR = "QDEVICE_DESTROY_ERROR"
QDEVICE_DESTROY_SUCCESS = "QDEVICE_DESTROY_SUCCESS"
QDEVICE_GET_STATUS_ERROR = "QDEVICE_GET_STATUS_ERROR"
QDEVICE_INITIALIZATION_ERROR = "QDEVICE_INITIALIZATION_ERROR"
QDEVICE_INITIALIZATION_SUCCESS = "QDEVICE_INITIALIZATION_SUCCESS"
QDEVICE_NOT_DEFINED = "QDEVICE_NOT_DEFINED"
QDEVICE_NOT_INITIALIZED = "QDEVICE_NOT_INITIALIZED"
QDEVICE_NOT_RUNNING = "QDEVICE_NOT_RUNNING"
QDEVICE_CLIENT_RELOAD_STARTED = "QDEVICE_CLIENT_RELOAD_STARTED"
QDEVICE_REMOVE_OR_CLUSTER_STOP_NEEDED = "QDEVICE_REMOVE_OR_CLUSTER_STOP_NEEDED"
QDEVICE_USED_BY_CLUSTERS = "QDEVICE_USED_BY_CLUSTERS"
REQUIRED_OPTIONS_ARE_MISSING = "REQUIRED_OPTIONS_ARE_MISSING"
REQUIRED_OPTION_OF_ALTERNATIVES_IS_MISSING = "REQUIRED_OPTION_OF_ALTERNATIVES_IS_MISSING"
RESOURCE_BAN_PCMK_ERROR = "RESOURCE_BAN_PCMK_ERROR"
RESOURCE_BAN_PCMK_SUCCESS = "RESOURCE_BAN_PCMK_SUCCESS"
RESOURCE_BUNDLE_ALREADY_CONTAINS_A_RESOURCE = "RESOURCE_BUNDLE_ALREADY_CONTAINS_A_RESOURCE"
RESOURCE_BUNDLE_UNSUPPORTED_CONTAINER_TYPE = "RESOURCE_BUNDLE_UNSUPPORTED_CONTAINER_TYPE"
RESOURCE_CLEANUP_ERROR = "RESOURCE_CLEANUP_ERROR"
RESOURCE_DOES_NOT_RUN = "RESOURCE_DOES_NOT_RUN"
RESOURCE_DISABLE_AFFECTS_OTHER_RESOURCES = "RESOURCE_DISABLE_AFFECTS_OTHER_RESOURCES"
RESOURCE_FOR_CONSTRAINT_IS_MULTIINSTANCE = 'RESOURCE_FOR_CONSTRAINT_IS_MULTIINSTANCE'
RESOURCE_IN_BUNDLE_NOT_ACCESSIBLE = "RESOURCE_IN_BUNDLE_NOT_ACCESSIBLE"
RESOURCE_INSTANCE_ATTR_VALUE_NOT_UNIQUE = "RESOURCE_INSTANCE_ATTR_VALUE_NOT_UNIQUE"
RESOURCE_IS_GUEST_NODE_ALREADY = "RESOURCE_IS_GUEST_NODE_ALREADY"
RESOURCE_IS_UNMANAGED = "RESOURCE_IS_UNMANAGED"
RESOURCE_MANAGED_NO_MONITOR_ENABLED = "RESOURCE_MANAGED_NO_MONITOR_ENABLED"
RESOURCE_MOVE_PCMK_ERROR = "RESOURCE_MOVE_PCMK_ERROR"
RESOURCE_MOVE_PCMK_SUCCESS = "RESOURCE_MOVE_PCMK_SUCCESS"
RESOURCE_OPERATION_INTERVAL_ADAPTED = "RESOURCE_OPERATION_INTERVAL_ADAPTED"
RESOURCE_OPERATION_INTERVAL_DUPLICATION = "RESOURCE_OPERATION_INTERVAL_DUPLICATION"
RESOURCE_REFRESH_ERROR = "RESOURCE_REFRESH_ERROR"
RESOURCE_REFRESH_TOO_TIME_CONSUMING = 'RESOURCE_REFRESH_TOO_TIME_CONSUMING'
RESOURCE_RUNNING_ON_NODES = "RESOURCE_RUNNING_ON_NODES"
RESOURCE_UNMOVE_UNBAN_PCMK_ERROR = "RESOURCE_UNMOVE_UNBAN_PCMK_ERROR"
RESOURCE_UNMOVE_UNBAN_PCMK_SUCCESS = "RESOURCE_UNMOVE_UNBAN_PCMK_SUCCESS"
RESOURCE_UNMOVE_UNBAN_PCMK_EXPIRED_NOT_SUPPORTED = "RESOURCE_UNMOVE_UNBAN_PCMK_EXPIRED_NOT_SUPPORTED"
RUN_EXTERNAL_PROCESS_ERROR = "RUN_EXTERNAL_PROCESS_ERROR"
RUN_EXTERNAL_PROCESS_FINISHED = "RUN_EXTERNAL_PROCESS_FINISHED"
RUN_EXTERNAL_PROCESS_STARTED = "RUN_EXTERNAL_PROCESS_STARTED"
SBD_CHECK_STARTED = "SBD_CHECK_STARTED"
SBD_CHECK_SUCCESS = "SBD_CHECK_SUCCESS"
SBD_CONFIG_ACCEPTED_BY_NODE = "SBD_CONFIG_ACCEPTED_BY_NODE"
SBD_CONFIG_DISTRIBUTION_STARTED = "SBD_CONFIG_DISTRIBUTION_STARTED"
SBD_DEVICE_DOES_NOT_EXIST = "SBD_DEVICE_DOES_NOT_EXIST"
SBD_DEVICE_DUMP_ERROR = "SBD_DEVICE_DUMP_ERROR"
SBD_DEVICE_INITIALIZATION_ERROR = "SBD_DEVICE_INITIALIZATION_ERROR"
SBD_DEVICE_INITIALIZATION_STARTED = "SBD_DEVICE_INITIALIZATION_STARTED"
SBD_DEVICE_INITIALIZATION_SUCCESS = "SBD_DEVICE_INITIALIZATION_SUCCESS"
SBD_DEVICE_IS_NOT_BLOCK_DEVICE = "SBD_DEVICE_IS_NOT_BLOCK_DEVICE"
SBD_DEVICE_LIST_ERROR = "SBD_DEVICE_LIST_ERROR"
SBD_DEVICE_MESSAGE_ERROR = "SBD_DEVICE_MESSAGE_ERROR"
SBD_DEVICE_PATH_NOT_ABSOLUTE = "SBD_DEVICE_PATH_NOT_ABSOLUTE"
SBD_DISABLING_STARTED = "SBD_DISABLING_STARTED"
SBD_ENABLING_STARTED = "SBD_ENABLING_STARTED"
SBD_LIST_WATCHDOG_ERROR = "SBD_LIST_WATCHDOG_ERROR"
SBD_NO_DEVICE_FOR_NODE = "SBD_NO_DEVICE_FOR_NODE"
SBD_NOT_INSTALLED = "SBD_NOT_INSTALLED"
SBD_NOT_USED_CANNOT_SET_SBD_OPTIONS = "SBD_NOT_USED_CANNOT_SET_SBD_OPTIONS"
SBD_TOO_MANY_DEVICES_FOR_NODE = "SBD_TOO_MANY_DEVICES_FOR_NODE"
SBD_WITH_DEVICES_NOT_USED_CANNOT_SET_DEVICE = "SBD_WITH_DEVICES_NOT_USED_CANNOT_SET_DEVICE"
SBD_WATCHDOG_NOT_SUPPORTED = "SBD_WATCHDOG_NOT_SUPPORTED"
SBD_WATCHDOG_VALIDATION_INACTIVE = "SBD_WATCHDOG_VALIDATION_INACTIVE"
SBD_WATCHDOG_TEST_ERROR = "SBD_WATCHDOG_TEST_ERROR"
SBD_WATCHDOG_TEST_MULTUPLE_DEVICES = "SBD_WATCHDOG_TEST_MULTUPLE_DEVICES"
SBD_WATCHDOG_TEST_FAILED = "SBD_WATCHDOG_TEST_FAILED"
SERVICE_DISABLE_ERROR = "SERVICE_DISABLE_ERROR"
SERVICE_DISABLE_STARTED = "SERVICE_DISABLE_STARTED"
SERVICE_DISABLE_SUCCESS = "SERVICE_DISABLE_SUCCESS"
SERVICE_ENABLE_ERROR = "SERVICE_ENABLE_ERROR"
SERVICE_ENABLE_STARTED = "SERVICE_ENABLE_STARTED"
SERVICE_ENABLE_SKIPPED = "SERVICE_ENABLE_SKIPPED"
SERVICE_ENABLE_SUCCESS = "SERVICE_ENABLE_SUCCESS"
SERVICE_KILL_ERROR = "SERVICE_KILL_ERROR"
SERVICE_KILL_SUCCESS = "SERVICE_KILL_SUCCESS"
SERVICE_NOT_INSTALLED = "SERVICE_NOT_INSTALLED"
SERVICE_START_ERROR = "SERVICE_START_ERROR"
SERVICE_START_SKIPPED = "SERVICE_START_SKIPPED"
SERVICE_START_STARTED = "SERVICE_START_STARTED"
SERVICE_START_SUCCESS = "SERVICE_START_SUCCESS"
SERVICE_STOP_ERROR = "SERVICE_STOP_ERROR"
SERVICE_STOP_STARTED = "SERVICE_STOP_STARTED"
SERVICE_STOP_SUCCESS = "SERVICE_STOP_SUCCESS"
SERVICE_VERSION_MISMATCH = "SERVICE_VERSION_MISMATCH"
STONITH_RESOURCES_DO_NOT_EXIST = "STONITH_RESOURCES_DO_NOT_EXIST"
SERVICE_COMMANDS_ON_NODES_STARTED = "SERVICE_COMMANDS_ON_NODES_STARTED"
SERVICE_COMMANDS_ON_NODES_SKIPPED = "SERVICE_COMMANDS_ON_NODES_SKIPPED"
SERVICE_COMMAND_ON_NODE_ERROR = "SERVICE_COMMAND_ON_NODE_ERROR"
SERVICE_COMMAND_ON_NODE_SUCCESS = "SERVICE_COMMAND_ON_NODE_SUCCESS"
SYSTEM_WILL_RESET = "SYSTEM_WILL_RESET"
TMP_FILE_WRITE = "TMP_FILE_WRITE"
UNABLE_TO_CONNECT_TO_ANY_REMAINING_NODE = "UNABLE_TO_CONNECT_TO_ANY_REMAINING_NODE"
UNABLE_TO_CONNECT_TO_ALL_REMAINING_NODE = "UNABLE_TO_CONNECT_TO_ALL_REMAINING_NODE"
UNABLE_TO_DETERMINE_USER_UID = "UNABLE_TO_DETERMINE_USER_UID"
UNABLE_TO_DETERMINE_GROUP_GID = "UNABLE_TO_DETERMINE_GROUP_GID"
UNABLE_TO_GET_AGENT_METADATA = 'UNABLE_TO_GET_AGENT_METADATA'
UNABLE_TO_READ_COROSYNC_CONFIG = "UNABLE_TO_READ_COROSYNC_CONFIG"
UNABLE_TO_GET_SBD_CONFIG = "UNABLE_TO_GET_SBD_CONFIG"
UNABLE_TO_GET_SBD_STATUS = "UNABLE_TO_GET_SBD_STATUS"
UNABLE_TO_PERFORM_OPERATION_ON_ANY_NODE = "UNABLE_TO_PERFORM_OPERATION_ON_ANY_NODE"
WATCHDOG_INVALID = "WATCHDOG_INVALID"
UNSUPPORTED_OPERATION_ON_NON_SYSTEMD_SYSTEMS = "UNSUPPORTED_OPERATION_ON_NON_SYSTEMD_SYSTEMS"
USE_COMMAND_NODE_ADD_REMOTE = "USE_COMMAND_NODE_ADD_REMOTE"
USE_COMMAND_NODE_ADD_GUEST = "USE_COMMAND_NODE_ADD_GUEST"
USE_COMMAND_NODE_REMOVE_GUEST = "USE_COMMAND_NODE_REMOVE_GUEST"
USING_KNOWN_HOST_ADDRESS_FOR_HOST = "USING_KNOWN_HOST_ADDRESS_FOR_HOST"
USING_DEFAULT_WATCHDOG = "USING_DEFAULT_WATCHDOG"
WAIT_FOR_IDLE_ERROR = "WAIT_FOR_IDLE_ERROR"
WAIT_FOR_IDLE_NOT_LIVE_CLUSTER = "WAIT_FOR_IDLE_NOT_LIVE_CLUSTER"
WAIT_FOR_IDLE_NOT_SUPPORTED = "WAIT_FOR_IDLE_NOT_SUPPORTED"
WAIT_FOR_IDLE_TIMED_OUT = "WAIT_FOR_IDLE_TIMED_OUT"
WAIT_FOR_NODE_STARTUP_ERROR = "WAIT_FOR_NODE_STARTUP_ERROR"
WAIT_FOR_NODE_STARTUP_STARTED = "WAIT_FOR_NODE_STARTUP_STARTED"
WAIT_FOR_NODE_STARTUP_TIMED_OUT = "WAIT_FOR_NODE_STARTUP_TIMED_OUT"
WAIT_FOR_NODE_STARTUP_WITHOUT_START = "WAIT_FOR_NODE_STARTUP_WITHOUT_START"
WATCHDOG_NOT_FOUND = "WATCHDOG_NOT_FOUND"
